@@ -3,6 +3,8 @@
 #endif
 
 #include "gtk-clutter-util.h"
+#include "gtk-clutter-offscreen.h"
+#include "gtk-clutter-version.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdk.h>
@@ -23,6 +25,7 @@
 
 /**
  * SECTION:gtk-clutter-util
+ * @Title: Utility Functions
  * @short_description: Utility functions for integrating Clutter in GTK+
  *
  * In order to properly integrate a Clutter scene into a GTK+ applications
@@ -33,6 +36,10 @@
  * GTK+ stock items and icon themes.
  */
 
+const guint clutter_gtk_major_version = CLUTTER_GTK_MAJOR_VERSION;
+const guint clutter_gtk_minor_version = CLUTTER_GTK_MINOR_VERSION;
+const guint clutter_gtk_micro_version = CLUTTER_GTK_MICRO_VERSION;
+
 enum
 {
   /* base symbols from GtkRcStyle */
@@ -106,12 +113,10 @@ gtk_clutter_get_component (GtkWidget    *widget,
  * gtk_clutter_get_fg_color:
  * @widget: a #GtkWidget
  * @state: a state
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the foreground color of @widget for the given @state and copies
  * it into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_fg_color (GtkWidget    *widget,
@@ -130,12 +135,10 @@ gtk_clutter_get_fg_color (GtkWidget    *widget,
  * gtk_clutter_get_bg_color:
  * @widget: a #GtkWidget
  * @state: a state
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the background color of @widget for the given @state and copies
  * it into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_bg_color (GtkWidget    *widget,
@@ -154,12 +157,10 @@ gtk_clutter_get_bg_color (GtkWidget    *widget,
  * gtk_clutter_get_text_color:
  * @widget: a #GtkWidget
  * @state: a state
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the text color of @widget for the given @state and copies it
  * into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_text_color (GtkWidget    *widget,
@@ -178,12 +179,10 @@ gtk_clutter_get_text_color (GtkWidget    *widget,
  * gtk_clutter_get_base_color:
  * @widget: a #GtkWidget
  * @state: a state
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the base color of @widget for the given @state and copies it
  * into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_base_color (GtkWidget    *widget,
@@ -202,12 +201,10 @@ gtk_clutter_get_base_color (GtkWidget    *widget,
  * gtk_clutter_get_light_color:
  * @widget: a #GtkWidget
  * @state: a state
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the light color of @widget for the given @state and copies it
  * into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_light_color (GtkWidget    *widget,
@@ -226,12 +223,10 @@ gtk_clutter_get_light_color (GtkWidget    *widget,
  * gtk_clutter_get_mid_color:
  * @widget: a #GtkWidget
  * @state: a state
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the mid color of @widget for the given @state and copies it
  * into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_mid_color (GtkWidget    *widget,
@@ -250,12 +245,10 @@ gtk_clutter_get_mid_color (GtkWidget    *widget,
  * gtk_clutter_get_dark_color:
  * @widget: a #GtkWidget
  * @state: a state
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the dark color of @widget for the given @state and copies it
  * into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_dark_color (GtkWidget    *widget,
@@ -278,8 +271,6 @@ gtk_clutter_get_dark_color (GtkWidget    *widget,
  *
  * Retrieves the text-aa color of @widget for the given @state and copies it
  * into @color.
- *
- * Since: 0.8
  */
 void
 gtk_clutter_get_text_aa_color (GtkWidget    *widget,
@@ -302,8 +293,6 @@ gtk_clutter_get_text_aa_color (GtkWidget    *widget,
  * of @pixbuf.
  *
  * Return value: the newly created #ClutterTexture
- *
- * Since: 0.8
  */
 ClutterActor *
 gtk_clutter_texture_new_from_pixbuf (GdkPixbuf *pixbuf)
@@ -349,8 +338,6 @@ gtk_clutter_texture_error_quark (void)
  * Sets the contents of @texture with a copy of @pixbuf.
  *
  * Return value: %TRUE on success, %FALSE on failure.
- *
- * Since: 0.8
  */
 gboolean
 gtk_clutter_texture_set_from_pixbuf (ClutterTexture *texture,
@@ -381,8 +368,6 @@ gtk_clutter_texture_set_from_pixbuf (ClutterTexture *texture,
  * icon @stock_id as rendered by @widget.
  *
  * Return value: the newly created #ClutterTexture
- *
- * Since: 0.8
  */
 ClutterActor *
 gtk_clutter_texture_new_from_stock (GtkWidget   *widget,
@@ -420,8 +405,6 @@ gtk_clutter_texture_new_from_stock (GtkWidget   *widget,
  * rendered by @widget.
  *
  * Return value: %TRUE on success, %FALSE on failure.
- *
- * Since: 0.8
  */
 gboolean
 gtk_clutter_texture_set_from_stock (ClutterTexture *texture,
@@ -442,8 +425,8 @@ gtk_clutter_texture_set_from_stock (ClutterTexture *texture,
   if (!pixbuf)
     {
       g_set_error (error,
-                   CLUTTER_GTK_TEXTURE_ERROR,
-                   CLUTTER_GTK_TEXTURE_INVALID_STOCK_ID,
+                   GTK_CLUTTER_TEXTURE_ERROR,
+                   GTK_CLUTTER_TEXTURE_INVALID_STOCK_ID,
                    "Stock ID '%s' not found", stock_id);
       return FALSE;
     }
@@ -465,8 +448,6 @@ gtk_clutter_texture_set_from_stock (ClutterTexture *texture,
  *
  * Return value: the newly created texture, or %NULL if @widget
  *   was %NULL and @icon_name was not found.
- *
- * Since: 0.8
  */
 ClutterActor *
 gtk_clutter_texture_new_from_icon_name (GtkWidget   *widget,
@@ -543,8 +524,6 @@ gtk_clutter_texture_new_from_icon_name (GtkWidget   *widget,
  * current icon theme.
  *
  * Return value: %TRUE on success, %FALSE on failure.
- *
- * Since: 0.8
  */
 gboolean
 gtk_clutter_texture_set_from_icon_name (ClutterTexture *texture,
@@ -601,18 +580,92 @@ gtk_clutter_texture_set_from_icon_name (ClutterTexture *texture,
   return returnval;
 }
 
+static gboolean
+post_parse_hook (GOptionContext  *context,
+                 GOptionGroup    *group,
+                 gpointer         data,
+                 GError         **error)
+{
+#if defined(GDK_WINDOWING_X11)
+  /* share the X11 Display with GTK+ */
+  clutter_x11_set_display (GDK_DISPLAY());
+
+  /* let GTK+ in charge of the event handling */
+  clutter_x11_disable_event_retrieval ();
+#elif defined(GDK_WINDOWING_WIN32)
+  /* let GTK+ in charge of the event handling */
+  clutter_win32_disable_event_retrieval ();
+#endif /* GDK_WINDOWING_{X11,WIN32} */
+
+  /* this is required since parsing clutter's option group did not
+   * complete the initialization process
+   */
+  return clutter_init_with_args (NULL, NULL, NULL, NULL, NULL, error);
+}
+
+/**
+ * gtk_clutter_get_option_group:
+ *
+ * Returns a #GOptionGroup for the command line arguments recognized
+ * by Clutter. You should add this group to your #GOptionContext with
+ * g_option_context_add_group(), if you are using g_option_context_parse()
+ * to parse your commandline arguments instead of using gtk_clutter_init()
+ * or gtk_clutter_init_with_args().
+ *
+ * You should add this option group to your #GOptionContext after
+ * the GTK option group created with gtk_get_option_group(), and after
+ * the clutter option group obtained from clutter_get_option_group_without_init().
+ * You should not use clutter_get_option_group() together with this function.
+ *
+ * You must pass %TRUE to gtk_get_option_group() since gtk-clutter's option
+ * group relies on it.
+ *
+ * Parsing options using g_option_context_parse() with a #GOptionContext
+ * containing the returned #GOptionGroupwith will result in Clutter's and
+ * GTK-Clutter's initialisation.  That is, the following code:
+ *
+ * |[
+ *   g_option_context_add_group (context, gtk_get_option_group (TRUE));
+ *   g_option_context_add_group (context, cogl_get_option_group ());
+ *   g_option_context_add_group (context, clutter_get_option_group_without_init ());
+ *   g_option_context_add_group (context, gtk_clutter_get_option_group ());
+ *   res = g_option_context_parse (context, &amp;argc, &amp;argc, NULL);
+ * ]|
+ *
+ * is functionally equivalent to:
+ *
+ * |[
+ *   gtk_clutter_init (&amp;argc, &amp;argv);
+ * ]|
+ *
+ * After g_option_context_parse() on a #GOptionContext containing the
+ * the returned #GOptionGroup has returned %TRUE, Clutter and GTK-Clutter are
+ * guaranteed to be initialized.
+ *
+ * Return value: (transfer full): a #GOptionGroup for the commandline arguments
+ *   recognized by ClutterGtk
+ */
+GOptionGroup *
+gtk_clutter_get_option_group  (void)
+{
+  GOptionGroup *group;
+
+  group = g_option_group_new ("clutter-gtk", "", "", NULL, NULL);
+  g_option_group_set_parse_hooks (group, NULL, post_parse_hook);
+
+  return group;
+}
+
 /**
  * gtk_clutter_init:
  * @argc: pointer to the arguments count, or %NULL
- * @argv: pointer to the arguments vector, or %NULL
+ * @argv: (array length=argc): pointer to the arguments vector, or %NULL
  *
  * This function should be called instead of clutter_init() and
  * gtk_init().
  *
  * Return value: %CLUTTER_INIT_SUCCESS on success, a negative integer
  *   on failure.
- *
- * Since: 0.8
  */
 ClutterInitError
 gtk_clutter_init (int    *argc,
@@ -622,9 +675,18 @@ gtk_clutter_init (int    *argc,
     return CLUTTER_INIT_ERROR_GTK;
 
 #if defined(HAVE_CLUTTER_GTK_X11)
-  clutter_x11_set_display (gdk_x11_display_get_xdisplay (gdk_display_get_default ()));
+# if CLUTTER_CHECK_VERSION (1, 1, 5)
+  /* enable ARGB visuals by default for Clutter */
+  clutter_x11_set_use_argb_visual (TRUE);
+# endif
+
+  /* share the X11 Display with GTK+ */
+  clutter_x11_set_display (GDK_DISPLAY());
+
+  /* let GTK+ in charge of the event handling */
   clutter_x11_disable_event_retrieval ();
 #elif defined(HAVE_CLUTTER_GTK_WIN32)
+  /* let GTK+ in charge of the event handling */
   clutter_win32_disable_event_retrieval ();
 #endif /* HAVE_CLUTTER_GTK_{X11,WIN32} */
 
@@ -634,7 +696,7 @@ gtk_clutter_init (int    *argc,
 /**
  * gtk_clutter_init_with_args:
  * @argc: a pointer to the number of command line arguments.
- * @argv: a pointer to the array of command line arguments.
+ * @argv: (array length=argc): a pointer to the array of command line arguments.
  * @parameter_string: a string which is displayed in
  *    the first line of <option>--help</option> output, after
  *    <literal><replaceable>programname</replaceable> [OPTION...]</literal>
@@ -650,8 +712,6 @@ gtk_clutter_init (int    *argc,
  *
  * Return value: %CLUTTER_INIT_SUCCESS on success, a negative integer
  *   on failure.
- *
- * Since: 0.10
  */
 ClutterInitError
 gtk_clutter_init_with_args (int            *argc,
@@ -661,20 +721,31 @@ gtk_clutter_init_with_args (int            *argc,
                             const char     *translation_domain,
                             GError        **error)
 {
-  GOptionGroup *gtk_group, *clutter_group;
+  GOptionGroup *gtk_group, *clutter_group, *cogl_group, *clutter_gtk_group;
   GOptionContext *context;
   gboolean res;
 
+#if defined(GDK_WINDOWING_X11) && CLUTTER_CHECK_VERSION (1, 1, 5)
+  /* enable ARGB visuals by default for Clutter */
+  clutter_x11_set_use_argb_visual (TRUE);
+#endif
+
   /* we let gtk+ open the display */
   gtk_group = gtk_get_option_group (TRUE);
 
   /* and we prevent clutter from doing so too */
   clutter_group = clutter_get_option_group_without_init ();
 
+  cogl_group = cogl_get_option_group ();
+
+  clutter_gtk_group = gtk_clutter_get_option_group ();
+
   context = g_option_context_new (parameter_string);
 
   g_option_context_add_group (context, gtk_group);
+  g_option_context_add_group (context, cogl_group);
   g_option_context_add_group (context, clutter_group);
+  g_option_context_add_group (context, clutter_gtk_group);
 
   if (entries)
     g_option_context_add_main_entries (context, entries, translation_domain);
@@ -685,19 +756,5 @@ gtk_clutter_init_with_args (int            *argc,
   if (!res)
     return CLUTTER_INIT_ERROR_GTK;
 
-#if defined(GDK_WINDOWING_X11)
-  clutter_x11_set_display (gdk_x11_display_get_xdisplay (gdk_display_get_default ()));
-  clutter_x11_disable_event_retrieval ();
-#elif defined(GDK_WINDOWING_WIN32)
-  clutter_win32_disable_event_retrieval ();
-#endif /* GDK_WINDOWING_{X11,WIN32} */
-
-  /* this is required since parsing clutter's option group did not
-   * complete the initialization process
-   */
-  return clutter_init_with_args (argc, argv,
-                                 NULL,
-                                 NULL,
-                                 NULL,
-                                 error);
+  return CLUTTER_INIT_SUCCESS;
 }
